/*
 * @Author       : Eleanor Mao
 * @Date         : 2024-04-09 10:36:44
 * @LastEditTime : 2024-04-09 10:36:44
 *
 *
 */
import React, { ChangeEvent, KeyboardEvent, useEffect, useMemo, useRef, useState } from "react";
import Grid from '@mui/material/Grid';
import Box from '@mui/material/Box';
import Button from '@mui/material/Button';
import TextField from '@mui/material/TextField';
import OutlinedInput from '@mui/material/OutlinedInput';
import SendIcon from '@mui/icons-material/Send';
import List from '@mui/material/List';
import Chip from '@mui/material/Chip';
import Divider from '@mui/material/Divider';
import { ChatItem } from "./ChatItem";
import Stack from '@mui/material/Stack';
import { uniqueId } from 'lodash';
import Typography from "@mui/material/Typography";
import { useLocation } from "react-router-dom";
import axios from 'axios';
import { Definition, DSL2Json, Json2Preview } from "./loadYml";
import RefreshIcon from '@mui/icons-material/Refresh';
import IconButton from '@mui/material/IconButton';
import Tooltip from '@mui/material/Tooltip';

interface Message {
  id: string;
  message: string;
  isMe: boolean;
  loading?: boolean;
}

const SupportedBlock = ['BotMessage', 'Chat', 'GlipSender', 'Script'];

export const Create = () => {
  const location = useLocation();
  const id = useMemo(() => new URLSearchParams(location.search).get('id'), [location]);
  const [formJson, setFormJson] = useState(() => {
    let value = {
      id: 'abc',
      description: '',
      name: 'My workflow',
      versionId: '',
    };
    try {
      value = JSON.parse(`create-${id}`);
    } catch (e) {
    }
    return value;
  });
  const [chatList, setChatList] = useState<Message[]>([{
    id: uniqueId('r'),
    message: [
      'Hello! Please tell me what kind of workflow you want to generate!\n',
      'Currently the node types I support are: Chat, BotMessage, GlipSender, Script'].join('\n'),
    isMe: false
  }]);
  const [chatInput, setChatInput] = useState('');
  const [workflowContent, setWorkflowContent] = useState('');
  // eslint-disable-next-line no-template-curly-in-string
  const flowDefinition = useMemo(() => DSL2Json(workflowContent), [workflowContent]);
  const previewImg = useMemo(() => Json2Preview(flowDefinition), [flowDefinition]);
  const chatContentRef = useRef<HTMLUListElement>(null);
  const hasEmailSender = useMemo(() => workflowContent.includes('"EmailSender"'), [workflowContent]);
  const hasBotAddin = useMemo(() => workflowContent.includes('"Chat"'), [workflowContent]);
  const hasScript = useMemo(() => workflowContent.includes('"Script"'), [workflowContent]);

  const [emailData, setEmailData] = useState({
    to: '',
    subject: '',
    content: `${content}`
  });
  useEffect(() => {
    console.log(workflowContent);
    console.log(flowDefinition);
  }, [flowDefinition]);
  // useEffect(() => {
  //   axios.post('/chat/start', {
  //     accountId: 10086,
  //     channelId: "10000",
  //     dialogId: "1",
  //     segmentId: "1"
  //   });
  // }, []);
  const handleChange = (e: ChangeEvent<HTMLInputElement>) => {
    setChatInput(e.target.value);
  };

  const handleClickChip = (text: string) => () => {
    setChatInput(s => s + text);
  };
  const handleSend = async () => {
    const rId = uniqueId('r');

    setChatList([...chatList, {
      id: uniqueId('m'),
      message: chatInput,
      isMe: true
    }, {
      id: rId,
      message: '',
      isMe: false,
      loading: true
    }]);
    setChatInput('');
    try {
      const { data } = await axios.post<{ answer: string; workflowContent: string }>(`/bot/workflow/${id}/messages`, {
        "question": chatInput
      });
      setWorkflowContent(data.workflowContent);
      setChatList(list => list.map(l => {
        if (l.id === rId) {
          l.loading = false;
          l.message = data.answer;
        }
        return l;
      }));
    } catch (e) {
      setChatList(list => list.filter(l => l.id !== rId));
    }
  };

  useEffect(() => {
    if (chatContentRef.current) {
      chatContentRef.current.scrollTop = chatContentRef.current.scrollHeight - chatContentRef.current.offsetHeight;
    }
  }, [chatList]);

  const handleRefresh = async () => {
    await axios.delete(`/bot/workflow/${id}/messages`);
    setChatList([{
      id: uniqueId('r'),
      message: [
        'Hello! Please tell me what kind of workflow you want to generate!\n',
        'Currently the node types I support are: Chat, BotMessage, GlipSender, Script'].join('\n'),
      isMe: false
    }]);
    setChatInput('');
    setWorkflowContent('');
  };

  return (
    <>
      <Grid container spacing={2}>
        <Grid item xs={12}>
          <TextField value={formJson.name} fullWidth variant="outlined" label="Workflow Name"
                     placeholder="Please name your workflow" onChange={e => {
            setFormJson(json => ({
              ...json,
              name: e.target.value
            }));
          }}/>

        </Grid>
        <Grid item xs={12}>
          <Grid container spacing={2}>
            <Grid item xs>
              <Box sx={{
                border: 1,
                height: 600,
                borderRadius: 1,
                mt: 4,
                borderColor: 'rgba(255,255,255,0.23)',
                position: 'relative'
              }}>
                <Tooltip title="Refresh AI">
                  <IconButton onClick={handleRefresh} style={{ position: 'absolute', right: 10, top: 10, zIndex: 5 }}>
                    <RefreshIcon/>
                  </IconButton>
                </Tooltip>
                <div style={{ height: '100%', display: 'flex', flexDirection: 'column' }}>
                  <List ref={chatContentRef}
                        style={{ flex: '1 1 auto', overflowY: 'auto', overflowX: 'hidden' }}
                        sx={{ p: 2 }}>
                    {chatList.map(chat => <ChatItem isMe={chat.isMe} message={chat.message} key={chat.id}
                                                    loading={chat.loading}/>)}
                  </List>
                  <Box sx={{ display: 'flex', flex: '0 0 auto', p: 2, borderTop: 1, borderColor: 'divider' }}>
                    <Grid container spacing={2}>
                      <Grid item xs={12} style={{ gap: 8 }}>
                        <Stack direction="row" spacing={1}>
                          <Typography variant="body2">
                            Supported Action Types:
                          </Typography>
                          {SupportedBlock.map(label => (
                            <Chip label={label} key={label} size="small" clickable color="primary"
                                  onClick={handleClickChip(label)}/>))}
                        </Stack>
                      </Grid>
                      <Grid item xs>
                        <OutlinedInput fullWidth size="small" onChange={handleChange}
                                       onKeyDown={(e: KeyboardEvent<HTMLInputElement>) => {
                                         if (e.key === 'Enter' && chatInput.length) {
                                           e.preventDefault();
                                           handleSend();
                                         }
                                       }}
                                       autoFocus
                                       multiline maxRows={3}
                                       value={chatInput}/>
                      </Grid>
                      <Grid item xs="auto" justifyContent="center">
                        <Button variant="contained" size="large" disabled={!chatInput.length} onClick={handleSend}>
                          <SendIcon/>
                        </Button>
                      </Grid>
                    </Grid>
                  </Box>
                </div>
              </Box>
            </Grid>
            <Grid item xs={4}>
              <Box sx={{ border: 1, height: 600, borderRadius: 1, p: 2, mt: 4, borderColor: 'rgba(255,255,255,0.23)' }}>
                {!previewImg ? 'Workflow Preview...' :
                  <img src={previewImg} style={{ width: '100%', height: '100%' }} alt="preview"/>}
              </Box>
            </Grid>
          </Grid>
          <Divider sx={{ mt: 4, mb: 4 }}/>

        </Grid>
      </Grid>
      <Box
        sx={{
          position: 'fixed',
          bottom: 0,
          left: 0,
          right: 0,
          p: 2,
          boxShadow: 2,
          borderTop: 1,
          bgcolor: 'background.default',
          borderColor: 'divider',
          display: { xs: 'flex' }, justifyContent: "flex-end"
        }}>
        <Button variant="contained" style={{ width: 100 }}>Save</Button>
      </Box>
    </>
  );
};
