/*
 * @Author       : Eleanor Mao
 * @Date         : 2024-04-09 09:38:20
 * @LastEditTime : 2024-04-09 09:38:20
 *
 *
 */
import React, { useEffect, useState } from "react";
import Button from '@mui/material/Button';
import Box from '@mui/material/Box';
import { styled } from '@mui/material/styles';
import Typography from '@mui/material/Typography';
import CardActionArea from '@mui/material/CardActionArea';
import Switch from '@mui/material/Switch';
import { Link, useNavigate } from "react-router-dom";
import AddIcon from '@mui/icons-material/Add';
import Card from '@mui/material/Card';
import CardActions from '@mui/material/CardActions';
import CardContent from '@mui/material/CardContent';
import { RCIcon } from './RCIcon';
import axios from "axios";
import Container from '@mui/material/Container';

import Dialog from '@mui/material/Dialog';
import DialogTitle from '@mui/material/DialogTitle';
import DialogActions from '@mui/material/DialogActions';
import DialogContent from '@mui/material/DialogContent';
import DialogContentText from '@mui/material/DialogContentText';
import TextField from '@mui/material/TextField';

const Flex = styled(Box)(() => ({
  display: 'flex',
  flexWrap: 'wrap'
}));

interface Workflow {
  description: string;
  id: string;
  name: string;
}

export const List = () => {
  const [list, setList] = useState<Workflow[]>([]);
  const [showModal, setShowModal] = useState(false);

  const navigate = useNavigate();
  useEffect(() => {
    axios.get<{
      total: number
      workflows: Workflow[]
    }>('/api/public/wf/config/v1/account/10086/workflows').then(({ data }) => {
      setList(data.workflows);
    });
  }, []);
  const handleOpen = () => {
    setShowModal(true);
  };
  const handleClose = () => {
    setShowModal(false);
  };

  const handleSubmit = (event: React.FormEvent<HTMLFormElement>) => {
    event.preventDefault();
    const formData = new FormData(event.currentTarget);
    const formJson = Object.fromEntries((formData as any).entries());
    axios.post('/bot/workflow', { name: formJson.name }).then(({ data }) => {
      sessionStorage.setItem(`create-${data.visibleId}`, JSON.stringify({ ...data, name: formJson.name }));
      handleClose();
      navigate(`/create?id=${data.visibleId}`);
    });
  };
  return (
    <Container maxWidth="lg">
      <Flex style={{ justifyContent: 'flex-end' }}>
        <Button startIcon={<AddIcon/>} variant="contained" onClick={handleOpen}>Create Workflow</Button>
      </Flex>
      <Flex sx={{ gap: 4, mt: 4 }}>
        {list.map(item => (
          <Link
            key={item.id}
            to={`/detail/${item.id}`}
          >
            <Card sx={{ width: 263, borderRadius: 2, boxShadow: 2 }}>
              <CardActionArea>
                <CardContent>
                  <Typography gutterBottom variant="h5" component="div">
                    {item.name || 'My Workflow'}
                  </Typography>
                  <Box sx={{ minHeight: 60 }}>
                    <RCIcon/>
                  </Box>
                </CardContent>
                <CardActions
                  sx={{ display: { xs: 'flex' }, justifyContent: "flex-end", borderTop: 1, borderColor: 'divider' }}>
                  <Switch defaultChecked/>
                </CardActions>
              </CardActionArea>
            </Card>
          </Link>))}
      </Flex>
      <Dialog
        open={showModal}
        onClose={handleClose}
        PaperProps={{
          component: 'form',
          onSubmit: handleSubmit,
        }}
      >
        <DialogTitle>Create Workflow</DialogTitle>
        <DialogContent>
          <DialogContentText>
            To create a workflow, please name your workflow first.
          </DialogContentText>
          <TextField
            autoFocus
            required
            margin="dense"
            id="name"
            name="name"
            defaultValue="My workflow"
            label="Workflow Name"
            fullWidth
            focused
            variant="standard"
          />
        </DialogContent>
        <DialogActions>
          <Button onClick={handleClose}>Cancel</Button>
          <Button type="submit">Create</Button>
        </DialogActions>
      </Dialog>
    </Container>
  );
};
